//=============================================================================
// (c) Copyright 2005 Diamond Systems Corporation. Use of this source code
// is subject to the terms of Diamond Systems' Software License Agreement.
// Diamond Systems provides no warranty of proper performance if this
// source code is modified.
//
// File: Watchdog.c   v5.91
// Desc: Sample program that demonstrates how to use the watchdog
// Created by KL
//=============================================================================

#include <stdio.h>

#ifdef _WIN32
#ifndef _WIN32_WCE
#include <conio.h>
#endif

#include <windows.h>
#include <stdlib.h>
#include <math.h>
// diamond driver includes
#include "dscud.h"
#endif

// DOS
#ifdef __BORLANDC__
#include <conio.h>
#include <dos.h>
#include <stdlib.h>
#include <math.h>
// diamond driver includes
#include "../../../../current/dev/source/dscud.h"
#endif

// Linux and QNX
#if defined(linux) || defined(__QNXNTO__) || defined(_WRS_VXWORKS_5_X)
#include <stdlib.h>
#include <math.h>
#include <time.h>
// diamond driver includes
#include "dscud.h"


#ifdef _WRS_VXWORKS_5_X
#include <selectLib.h>
#define main ELEKTRAWatchdog
#else
#include <sys/time.h>
#endif

static int kbhit()
{
	struct timeval timeout;
	fd_set rfds;

	timeout.tv_sec = 0;
	timeout.tv_usec = 0;

	FD_ZERO(&rfds);
	FD_SET(0, &rfds);

	if ( select(0+1, &rfds, NULL, NULL, &timeout) > 0 )
		return getchar();

	return 0;
}
#endif

// var declarations
BYTE result; // returned error code
DSCB dscb;   // handle used to refer to the board
DSCCB dsccb; // structure containing board settings
ERRPARAMS errorParams;  // structure for returning error code and error string
float floatBuff;        // temp variable of size float
WORD FirstRebootTimer;  // firt timer program value
BYTE SecondRebootTimer; // second timer program value

//=============================================================================
// Name: main()
// Desc: Starting function that calls the driver functions used
//
//		 NOTE: By convention, you should capture the BYTE return value for each
//		 driver API call, and check the error code.
//
//     STEPS TO FOLLOW:
//
//	     I. Driver Initialization
//	    II. Board Initialization
//	   III. Watchdog Setup & Run
//	    IV. Cleanup
//
//=============================================================================

int main( void )
{
	//=========================================================================
	// I. DRIVER INITIALIZATION
	//
	//    Initializes the DSCUD library.
	//
	//    STEPS TO FOLLOW:
	//
	//	  1. initialize the driver, using the driver version for validation
	//=========================================================================

	if( dscInit( DSC_VERSION ) != DE_NONE )
	{
		dscGetLastError(&errorParams);
		fprintf( stderr, "dscInit error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
		return 0;
	}

	//=========================================================================
	// II. BOARD INITIALIZATION
	//
	//	   Initialize the ELEKTRA board. This function passes the various
	//	   hardware parameters to the driver and resets the hardware.
	//
	//     STEPS TO FOLLOW:
	//
	//	   1. set the board type to DSC_ELEKTRA for ELEKTRA board
	//	   2. set the base address to 0x280
	//	   3. set the interrupt level 5
	//	   4. intialize and register the board with the driver, after which
	//		  the struct, dscb, now holds the handle for the board
	//=========================================================================

	printf( "\nELEKTRA BOARD INITIALIZATION:\n" );

	dsccb.base_address = 0x280;

	dsccb.int_level = 5;

	if(dscInitBoard(DSC_ELEKTRA, &dsccb, &dscb)!= DE_NONE)
	{
		dscGetLastError(&errorParams);
		fprintf( stderr, "dscInitBoard error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
		return 0;
	}

	//=========================================================================
	// III. Watchdog Setup & Run
	//
	//	    Setup and enable the ELEKTRA Watchdog timer. Once the timer
	//		hits the threshold it will shutdown the ELEKTRA
	//
	//	    STEPS TO FOLLOW:
	//
	//	    1. set the 16-bit timer value as the second parameter
	//			NOTE: This timer can achieve a max of 2.0 seconds delay.
	//				  This timer runs at 32.768khz with 16-bit resolution 
	//	    2. set the 8-bit timer value as the third parameter
	//			NOTE: This timer can achieve a max of 7.2 milli-seconds delay
	//			      This timer runs at 32.768khz with 8-bit resolution
	//				  Also note that this timer is not precise and is a rough estimate.
	//	    3. set the watchdog timer option as the fourth parameter
	//			NOTE: The macros for this parameter are in DSCUD.H. Please 
	//			      refer to DSCUD.H for more details.
	//	    4. enable watchdog
	//=========================================================================

	printf("Enter value of first timer (max of 2.0 seconds, minimum of 0.0): ");
	scanf("%f", &floatBuff);
	FirstRebootTimer = (floatBuff/2.0f) * 65535L;  // Timer has a range of 0.0 sec to 2.0 sec,
										     // with a resolution of 2.0 s / 65535.0 or
										     // 0.0305 milli-seconds. 65535 is 0xFFFF.

	printf("Enter value of second timer (max of 7.2 ms, minimum of 0.0): ");
	scanf("%f", &floatBuff);
	SecondRebootTimer = (floatBuff/7.2f) * 255; // Timer has a range of 0.0 ms to 7.2 ms,
										   // with a resolution of 0.0305 milli-seconds.
										   

	if( dscWatchdogEnable(dscb, FirstRebootTimer, SecondRebootTimer, 0) != DE_NONE)
	{
		dscGetLastError(&errorParams);
		fprintf( stderr, "dscWatchdogEnable error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
		return 0;
	}


	// To disable watchdog uncomment the following code
	//
	// dscWatchdogDisable(dscb);
	//


	// To retrigger watchdog uncomment the following code
	//
	// NOTE: By retriggering the watchdog, all timers on the watchdog is reset
	// to the initial enabled values. In the case above, 
	// 0xFFFF for 16-bit timer (FirstRebootTimer), and 
	// 0xFF for 8-bit (SecondRebootTimer)
	//
	// dscWatchdogTrigger(dscb);
	//


	//=========================================================================
	// IV. CLEANUP
	//
	//	   Cleanup any remnants left by the program and free the resources used
	//	   by the driver.
	//
	//     STEPS TO FOLLOW:
	//
	//	   1. free the driver resources
	//=========================================================================

	dscFree();

	printf( "\nWatchdog completed.\n" );

	return 0;
} // end main()


#ifdef _WIN32_WCE
int WINAPI WinMain(	HINSTANCE hInstance,
					HINSTANCE hPrevInstance,
					LPTSTR    lpCmdLine,
					int       nCmdShow)
{
	main();

	return 0;
}

#endif
